@@ -34,6 +34,7 @@ class CourseraDownloader(object):
     :keyword proxy: http proxy, eg: foo.bar.com:1234
     :keyword parser: xml parser
     :keyword ignorefiles: comma separated list of file extensions to skip (e.g., "ppt,srt")
+    :keyword includefiles: comma separated list of file extensions to download (e.g., "pdf")
     """
     BASE_URL =    'https://class.coursera.org/%s'
     HOME_URL =    BASE_URL + '/class/index'
@@ -49,29 +50,47 @@ class CourseraDownloader(object):
     # how long to try to open a URL before timing out
     TIMEOUT=60.0
 
+    HTML_TEMPLATE = '''<!DOCTYPE html>
+    <html><head>
+        <meta charset="utf-8">
+        <title>Course materials</title>
+        <style type="text/css">
+          body { font-family: Sans-Serif; font-size: 90%%; }
+          a { font-size: 90%%; color: #07c; }
+          a:visited { color: #07c; }
+          h3 { color: #808080; }
+          div { color: #404040; margin: 10px; }
+        </style>
+    </head><body>
+    %s
+    </body></html>'''
+
 
     def __init__(self,username,
                         password,
                         proxy=None,
                         parser=DEFAULT_PARSER,
                         ignorefiles=None,
+                        includefiles=None,
                         max_path_part_len=None,
                         gzip_courses=False,
-                        wk_filter=None):
+                        wk_filter=None,
+                        lang=None):
 
         self.username = username
         self.password = password
         self.parser = parser
 
-        # Split "ignorefiles" argument on commas, strip, remove prefixing dot
-        # if there is one, and filter out empty tokens.
-        self.ignorefiles =  [x.strip()[1:] if x[0]=='.' else x.strip()
-                             for x in ignorefiles.split(',') if len(x)]
+        self.ignorefiles =  self.parseFileExtensions(ignorefiles)
+        self.includefiles = self.parseFileExtensions(includefiles)
 
         self.browser = None
         self.proxy = proxy
         self.max_path_part_len = max_path_part_len
         self.gzip_courses = gzip_courses
+        self.lang = lang
+
+        self.html = ""
 
         try:
             self.wk_filter = map(int,wk_filter.split(",")) if wk_filter else None
@@ -79,6 +98,15 @@ class CourseraDownloader(object):
             print "Invalid week filter, should be a comma separated list of integers", e
             exit()
 
+    @staticmethod
+    def parseFileExtensions(extensionsStr):
+        """
+        Split strings with file extensions ("ignorefiles" and "includefiles" arguments) on commas,
+        strip, remove prefixing dot if there is one, and filter out empty tokens.
+        """
+        return [x.strip()[1:] if x[0]=='.' else x.strip()
+                for x in extensionsStr.split(',') if len(x)]
+
     def login(self,className):
         """
         Login into coursera and obtain the necessary session cookies.
@@ -238,6 +266,14 @@ class CourseraDownloader(object):
                     if h.find('source_videos') > 0:
                         print "   - will skip raw source video " + h
                     else:
+                        if self.lang and h.find('subtitles') > 0:
+                            # Substitutes the matched language with user's one
+                            def language(match):
+                                num = match.group('num')
+                                return 'q={num}_{lang}'.format(num=num, lang=self.lang )
+
+                            h = re.sub('q=(?P<num>[\d]+)_\w+', language, h)
+
                         # Dont set a filename here, that will be inferred from the week
                         # titles
                         resourceLinks.append( (h,None) )
@@ -281,7 +317,7 @@ class CourseraDownloader(object):
         r = self.browser.open(url,timeout=self.TIMEOUT)
         return r.info()
 
-    def download(self, url, target_dir=".", target_fname=None):
+    def download(self, url, target_dir=".", target_fname=None, class_dir=None):
         """
         Download the url to the given filename
         """
@@ -306,8 +342,16 @@ class CourseraDownloader(object):
             print '    - skipping "%s" (extension ignored)' % fname
             return
 
+        # if downloading class resource (as opposed to lecture/syllabus pages), and '-i' arg specified
+        # then skip other file extensions (and files with no extensions)
+        if (class_dir and self.includefiles and not (ext and ext[1:] in self.includefiles)):
+            print '    - skipping "%s" (extension not included)' % fname
+            return
+
         filepath = trim_path(path.join(target_dir, fname), get_max_path_length()-1, 1)
 
+        if (class_dir): self.html += '<a href="%s">%s</a> \n' % (path.join(class_dir, fname), ext[1:])
+
         dl = True
         if path.exists(filepath):
             if clen > 0:
@@ -407,6 +451,7 @@ class CourseraDownloader(object):
         except Exception as e:
             print "Warning: failed to download about file",e
 
+
         # now download the actual content (video's, lecture notes, ...)
         for j, (weeklyTopic, weekClasses) in enumerate(weeklyTopics,start=1):
 
@@ -424,6 +469,7 @@ class CourseraDownloader(object):
                 os.makedirs(wkdir)
 
             print " - " + weeklyTopic
+            self.html += "<h3>%s</h3>\n" % weeklyTopic
 
             for i, (className, classResources) in enumerate(weekClasses,start=1):
 
@@ -438,13 +484,24 @@ class CourseraDownloader(object):
 
                 print "  - Downloading resources for " + className
 
+                self.html += "<div>%s<br>\n" % className
+
                 # download each resource
                 for classResource,tfname in classResources:
                     try:
-                       #print '  - Downloading ', classResource
-                       self.download(classResource,target_dir=clsdir,target_fname=tfname)
+                        print '    - Downloading ', classResource, tfname
+                        self.download(classResource,target_dir=clsdir,target_fname=tfname,
+                            class_dir=path.join(wkdirname, clsdirname))
                     except Exception as e:
-                       print "    - failed: ",classResource,e
+                        print "    - failed: ",classResource,e
+                self.html += "</div>\n"
+
+        try:
+            file = open(path.join(course_dir, 'materials.html'), "w")
+            file.write(self.HTML_TEMPLATE % self.html)
+            file.close()
+        except Exception as e:
+            print "  - Writing materials.html failed: ",e 
 
         if gzip_courses:
             tar_file_name = cname + ".tar.gz"
@@ -533,6 +590,8 @@ def main():
     parser.add_argument("-p", dest='password', type=str, help='coursera password')
     parser.add_argument("-d", dest='dest_dir', type=str, default=".", help='destination directory where everything will be saved')
     parser.add_argument("-n", dest='ignorefiles', type=str, default="", help='comma-separated list of file extensions to skip, e.g., "ppt,srt,pdf"')
+    parser.add_argument("-i", dest='includefiles', type=str, default="", help='comma-separated list of file extensions to download, e.g., "pdf,doc"')
+    parser.add_argument("-l", dest='lang', type=str, help='language of subtitles')
     parser.add_argument("-q", dest='parser', type=str, default=CourseraDownloader.DEFAULT_PARSER,
                         help="the html parser to use, see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser")
     parser.add_argument("-x", dest='proxy', type=str, default=None, help="proxy to use, e.g., foo.bar.com:3125")
@@ -577,9 +636,11 @@ def main():
                            proxy=args.proxy,
                            parser=html_parser,
                            ignorefiles=args.ignorefiles,
+                           includefiles=args.includefiles,
                            max_path_part_len=mppl,
                            gzip_courses=args.gzip_courses,
-                           wk_filter=args.wkfilter
+                           wk_filter=args.wkfilter,
+                           lang=args.lang,
                           )
 
     # authenticate, only need to do this once but need a classaname to get hold
